//
// Simple program to draw plots
//
#include <math.h>
#include <iostream>
#include <SDL2/SDL.h> // SDL2.0 library

SDL_Window* window = NULL;
SDL_Renderer* renderer = NULL;

// Prototypes:
double u(double x); // Function to plot
void w_plot(double *x, double *y, int n, double x0, double xsc, double ysc, uint col); // Draw plot
void w_axes(); // Draw axis
void w_clear(); // Clear screen

// Main function
int main(int argc, char* argv[]) {

  // Set a number of steps and a value of step
  int n = 10;
  std::cout << "n = ";
  std::cin >> n;
  double h = 1./n;

  // Initialize graphics
  SDL_CreateWindowAndRenderer(1024, 768, 0, &window, &renderer);
  w_clear();

  // Allocate arrayes
  double *u_x = new double[n+1];
  double *u_y = new double[n+1];

  // Set the mesh and calculate function u
  int i;
  for (i = 0; i <= n; i++) {
      u_x[i] = i*h;
      u_y[i] = u(u_x[i]);
  }

  // Draw plot
  w_axes();
  w_plot(u_x, u_y, n+1, 0., 1., 2., 1);

  // Stop
  std::cin.ignore(); std::cin.ignore();

  // Deallocate arrayes
  delete u_x;
  delete u_y;
  return 0; // Return
}

// Function to plot
double u(double x) {
  return sin(1./(x + .1)) + 1.;
}

// Draw plot
// x, y are arrayes of node coordinates
// n is the size of array
// x0 is the coordinate of zero point
// xsc, ysc are scales on the x, y axis
// col is a color of the plot
void w_plot(double *x, double *y, int n, double x0, double xsc, double ysc, uint col) {
  if (col == 0) { // Choose color for plot
    SDL_SetRenderDrawColor(renderer, 255, 0, 0, SDL_ALPHA_OPAQUE); // red
  } else if (col == 1) {
    SDL_SetRenderDrawColor(renderer, 0, 255, 0, SDL_ALPHA_OPAQUE); // green
  } else if (col == 2) {
    SDL_SetRenderDrawColor(renderer, 0, 0, 255, SDL_ALPHA_OPAQUE); // blue
  } else if (col == 3) {
    SDL_SetRenderDrawColor(renderer, 255, 255, 0, SDL_ALPHA_OPAQUE); // yellow
  } else if (col == 4) {
    SDL_SetRenderDrawColor(renderer, 255, 0, 255, SDL_ALPHA_OPAQUE); // magenta
  } else {
    SDL_SetRenderDrawColor(renderer, 0, 255, 255, SDL_ALPHA_OPAQUE); // cyan
  }

  double xa, xb;
  int i; // Main drawing loop
  for (i = 1; i < n; i++) {
    xa = (x[i-1] - x0)/xsc;
    xb = (x[i] - x0)/xsc;
    SDL_RenderDrawLine(renderer, 20 + xa*964, 748 - y[i-1]/ysc*708, \
                                 20 + xb*964, 748 - y[i]/ysc*708);
  }
  SDL_RenderPresent(renderer);
}

// Draw axis
void w_axes() {
  SDL_SetRenderDrawColor(renderer, 255, 255, 255, SDL_ALPHA_OPAQUE); // white
  SDL_RenderDrawLine(renderer, 20, 748, 1004, 748); // Draw x axis
  SDL_RenderDrawLine(renderer, 996, 744, 1004, 748);
  SDL_RenderDrawLine(renderer, 996, 752, 1004, 748);
  SDL_RenderDrawLine(renderer, 984, 744, 984, 752); // one tick W = 964
  SDL_RenderDrawLine(renderer, 20, 20, 20, 748); // Draw y axis
  SDL_RenderDrawLine(renderer, 20, 20, 24, 28);
  SDL_RenderDrawLine(renderer, 20, 20, 16, 28);
  SDL_RenderDrawLine(renderer, 16, 40, 24, 40); // one tick H = 708
  SDL_RenderPresent(renderer);
}

// Clear screen
void w_clear() {
  SDL_SetRenderDrawColor(renderer, 0, 0, 0, SDL_ALPHA_OPAQUE); // black
  SDL_RenderClear(renderer);
}